@@ -176,6 +176,9 @@ public:
 	bool is_14bit() const
 		{ return m_param.is_any() && m_value.is_14bit(); }
 
+	void clear_value()
+		{ m_value.clear(); }
+
 private:
 
 	unsigned short m_status;
@@ -327,8 +330,7 @@ public:
 
 		if (event.key.param == RPN_MSB) {
 			xrpn_item& item = get_item(channel);
-			if (item.is_param_msb()
-				|| (item.is_any() && item.type() != samplv1_controls::RPN))
+			if (item.is_any() && item.type() != samplv1_controls::RPN)
 				enqueue(item);
 			if (item.type() == samplv1_controls::None) {
 				item.set_status(samplv1_controls::RPN | channel);
@@ -340,8 +342,7 @@ public:
 		else
 		if (event.key.param == RPN_LSB) {
 			xrpn_item& item = get_item(channel);
-			if (item.is_param_lsb()
-				|| (item.is_any() && item.type() != samplv1_controls::RPN))
+			if (item.is_any() && item.type() != samplv1_controls::RPN)
 				enqueue(item);
 			if (item.type() == samplv1_controls::None) {
 				item.set_status(samplv1_controls::RPN | channel);
@@ -353,9 +354,19 @@ public:
 		else
 		if (event.key.param == NRPN_MSB) {
 			xrpn_item& item = get_item(channel);
-			if (item.is_param_msb()
-				|| (item.is_any() && item.type() != samplv1_controls::NRPN))
+			if (item.is_any() && item.type() != samplv1_controls::NRPN)
 				enqueue(item);
+			if (item.type() == samplv1_controls::RPN && item.is_param_lsb()
+				&& item.param_lsb() == 0x7f && event.value == 0x7f) {
+				item.clear();
+				--m_count;
+				return true;
+			}
+			if (item.type() == samplv1_controls::NRPN) {
+				item.clear();
+				item.set_status(samplv1_controls::RPN | channel);
+			}
+			else
 			if (item.type() == samplv1_controls::None) {
 				item.set_status(samplv1_controls::NRPN | channel);
 				++m_count;
@@ -366,9 +377,19 @@ public:
 		else
 		if (event.key.param == NRPN_LSB) {
 			xrpn_item& item = get_item(channel);
-			if (item.is_param_lsb()
-				|| (item.is_any() && item.type() != samplv1_controls::NRPN))
+			if (item.is_any() && item.type() != samplv1_controls::NRPN)
 				enqueue(item);
+			if (item.type() == samplv1_controls::RPN && item.is_param_msb()
+				&& item.param_msb() == 0x7f && event.value == 0x7f) {
+				item.clear();
+				--m_count;
+				return true;
+			}
+			if (item.type() == samplv1_controls::NRPN) {
+				item.clear();
+				item.set_status(samplv1_controls::RPN | channel);
+			}
+			else
 			if (item.type() == samplv1_controls::None) {
 				item.set_status(samplv1_controls::NRPN | channel);
 				++m_count;
@@ -379,11 +400,8 @@ public:
 		else
 		if (event.key.param == DATA_MSB) {
 			xrpn_item& item = get_item(channel);
-			if (item.type() == samplv1_controls::None)
-				return false;
-			if (item.is_value_msb()
-				|| (item.type() != samplv1_controls::RPN
-				 && item.type() != samplv1_controls::NRPN)) {
+			if (item.type() != samplv1_controls::RPN &&
+				item.type() != samplv1_controls::NRPN) {
 				enqueue(item);
 				return false;
 			}
@@ -395,11 +413,8 @@ public:
 		else
 		if (event.key.param == DATA_LSB) {
 			xrpn_item& item = get_item(channel);
-			if (item.type() == samplv1_controls::None)
-				return false;
-			if (item.is_value_lsb()
-				|| (item.type() != samplv1_controls::RPN
-				 && item.type() != samplv1_controls::NRPN)) {
+			if (item.type() != samplv1_controls::RPN &&
+				item.type() != samplv1_controls::NRPN) {
 				enqueue(item);
 				return false;
 			}
@@ -412,11 +427,21 @@ public:
 		if (event.key.param > CC14_MSB_MIN &&
 			event.key.param < CC14_MSB_MAX) {
 			xrpn_item& item = get_item(channel);
+			if (item.is_any() && item.type() != samplv1_controls::CC14) {
+				enqueue(item);
+				item.clear();
+				--m_count;
+			}
+			else
 			if (item.is_param_msb() || item.is_value_msb()
-				|| (item.is_any() && item.type() != samplv1_controls::CC14)
 				|| (item.type() == samplv1_controls::CC14
 					&& item.param_lsb() != event.key.param + CC14_LSB_MIN))
 				enqueue(item);
+			if (item.type() == samplv1_controls::RPN) {
+				item.clear();
+				item.set_status(samplv1_controls::NRPN | channel);
+			}
+			else
 			if (item.type() == samplv1_controls::None) {
 				item.set_status(samplv1_controls::CC14 | channel);
 				++m_count;
@@ -431,11 +456,21 @@ public:
 		if (event.key.param > CC14_LSB_MIN &&
 			event.key.param < CC14_LSB_MAX) {
 			xrpn_item& item = get_item(channel);
+			if (item.is_any() && item.type() != samplv1_controls::CC14) {
+				enqueue(item);
+				item.clear();
+				--m_count;
+			}
+			else
 			if (item.is_param_lsb() || item.is_value_lsb()
-				|| (item.is_any() && item.type() != samplv1_controls::CC14)
 				|| (item.type() == samplv1_controls::CC14
 					&& item.param_msb() != event.key.param - CC14_LSB_MIN))
 				enqueue(item);
+			if (item.type() == samplv1_controls::RPN) {
+				item.clear();
+				item.set_status(samplv1_controls::NRPN | channel);
+			}
+			else
 			if (item.type() == samplv1_controls::None) {
 				item.set_status(samplv1_controls::CC14 | channel);
 				++m_count;
@@ -471,10 +506,14 @@ protected:
 				if (item.is_value_lsb())
 					m_queue.push(status, item.param_lsb(), item.value_lsb());
 			}
+			item.clear();
+			--m_count;
 		}
 		else
 		if (item.is_ready()) {
 			m_queue.push(item.status(), item.param(), item.value());
+			item.clear_value();
+		//	--m_count;
 		} else {
 			const unsigned short status
 				= samplv1_controls::CC | item.channel();
@@ -495,10 +534,9 @@ protected:
 				m_queue.push(status, DATA_MSB, item.value_msb());
 			if (item.is_value_lsb())
 				m_queue.push(status, DATA_LSB, item.value_lsb());
+			item.clear();
+			--m_count;
 		}
-
-		item.clear();
-		--m_count;
 	}
 
 private:
@@ -603,13 +641,14 @@ void samplv1_controls::process_event ( const Event& event )
 
 	// catch-up testing begin...
 	bool bSync = (data.flags & Hook) || !samplv1_param::paramFloat(index);
-
+	if (!bSync)
+		bSync = data.sync;
 	if (!bSync) {
 		const float v0 = data.val;
 		const float v1 = samplv1_param::paramScale(index,
 			m_sched_in.instance()->paramValue(index));
 		const float d1 = (v1 - fScale);
-		const float d2 = (data.sync ? (v1 - v0) : d1) * d1;
+		const float d2 = (v1 - v0) * d1;
 		bSync = (d2 < 0.001f);
 		if (bSync) {
 			data.val = fScale;
